@@ -22,6 +22,7 @@ import inspect
 import distutils.sysconfig as sysconfig
 import warnings
 from os import path
+from pathlib import Path
 from fnmatch import fnmatch
 from abc import ABC, abstractmethod
 from typing import List, Optional, Union
@@ -116,14 +117,34 @@ class IgnoreDirname(IgnoreElem, attrs=['dirname']):
     def _post_init(self) -> None:
         # pylint: disable=access-member-before-definition
         # pylint: disable=attribute-defined-outside-init
+
+        # Path object will turn into str here
+        self.dirname = path.realpath(self.dirname)
+
         if not self.dirname.endswith(path.sep):
             self.dirname = f'{self.dirname}{path.sep}'
 
     def match(self, frame_no: int, frameinfos: List[inspect.FrameInfo]) -> bool:
         frame = frameinfos[frame_no].frame
+        filename = path.realpath(frame.f_code.co_filename)
 
-        return path.realpath(frame.f_code.co_filename).startswith(
-            path.realpath(self.dirname)
+        return filename.startswith(self.dirname)
+
+class IgnoreStdlib(IgnoreDirname, attrs=['dirname']):
+    """Ignore standard libraries in sysconfig.get_python_lib(standard_lib=True)
+
+    But we need to ignore 3rd-party packages under site-packages/.
+    """
+
+    def match(self, frame_no: int, frameinfos: List[inspect.FrameInfo]) -> bool:
+        frame = frameinfos[frame_no].frame
+        third_party_lib = f'{self.dirname}site-packages{path.sep}'
+        filename = path.realpath(frame.f_code.co_filename)
+
+        return (
+            filename.startswith(self.dirname) and
+            # Exclude 3rd-party libraries in site-packages
+            not filename.startswith(third_party_lib)
         )
 
 class IgnoreFunction(IgnoreElem, attrs=['func']):
@@ -229,7 +250,7 @@ def create_ignore_elem(ignore_elem: IgnoreElemType) -> IgnoreElem:
     """Create an ignore element according to the type"""
     if isinstance(ignore_elem, ModuleType):
         return IgnoreModule(ignore_elem)
-    if isinstance(ignore_elem, str):
+    if isinstance(ignore_elem, (Path, str)):
         return (IgnoreDirname(ignore_elem)
                 if path.isdir(ignore_elem)
                 else IgnoreFilename(ignore_elem))
@@ -249,7 +270,7 @@ def create_ignore_elem(ignore_elem: IgnoreElemType) -> IgnoreElem:
 
     if isinstance(ignore_elem[0], ModuleType):
         return IgnoreModuleQualname(*ignore_elem)
-    if isinstance(ignore_elem[0], str):
+    if isinstance(ignore_elem[0], (Path, str)):
         return IgnoreFilenameQualname(*ignore_elem)
     if ignore_elem[0] is None:
         return IgnoreOnlyQualname(*ignore_elem)
@@ -261,7 +282,8 @@ class IgnoreList:
     @classmethod
     def create(cls,
                ignore: Optional[IgnoreType] = None,
-               ignore_lambda: bool = True) -> "IgnoreList":
+               ignore_lambda: bool = True,
+               ignore_varname: bool = True) -> "IgnoreList":
         """Create an IgnoreList object
 
         Args:
@@ -270,6 +292,8 @@ class IgnoreList:
                 A tuple of module (or filename) and qualified name
                 A function
                 A tuple of function and number of decorators
+            ignore_lambda: whether ignore lambda functions
+            ignore_varname: whether the calls from this package
 
         Returns:
             The IgnoreList object
@@ -279,9 +303,10 @@ class IgnoreList:
             ignore = [ignore]
 
         ignore_list = [
-            create_ignore_elem(sysconfig.get_python_lib(standard_lib=True)),
-            create_ignore_elem(sys.modules[__package__])
+            IgnoreStdlib(sysconfig.get_python_lib(standard_lib=True))
         ]
+        if ignore_varname:
+            ignore_list.append(create_ignore_elem(sys.modules[__package__]))
         if ignore_lambda:
             ignore_list.append(create_ignore_elem((None, '*<lambda>')))
         for ignore_elem in ignore:
